// This file is part of Moonlight Creator
//   Copyright (C) 1996-1998  Stephane Rehel
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*
  MLFScaleXYZ.C

  Stephane Rehel
  August 2 1997
*/

#include "MLFScaleXYZ.h"

#include "tools/IntegerArray.h"

#include "model/ModelModule.h"
#include "model/ModelModes.h"
#include "model/ModelCommand.h"
#include "model/ModelCommands.h"
#include "model/ModelScene.h"

#include "model/commands/MLCScale.h"

#include "model/modes/MLMDrag.h"

/////////////////////////////////////////////////////////////////////////////

MLFScaleXYZ::MLFScaleXYZ( MLModule* _module ):
  MLBaseFunction(_module)
{}

/////////////////////////////////////////////////////////////////////////////

MLFScaleXYZ::~MLFScaleXYZ()
{}

/////////////////////////////////////////////////////////////////////////////

IBOOL MLFScaleXYZ::_call()
{
  module->enterMode(ModelModes::DRAG);

  ModelModule* model= (ModelModule*) module;

  model->pushMessage( "Scale: L(X)  M(Y)  R(Z)" );

  while( ! exitFunction )
    {
    if( ! interfaceYield() )
      break;

    MLMDrag* d= (MLMDrag*) model->getMode(ModelModes::DRAG);
    if( d == 0 )
      break;

    MLCScale* cmd= new MLCScale;
    cmd->lonelyCommand= 0;

    model->getModelScene()->getSelectedObjects(cmd);

    if( cmd->indices.getSize() == 0 )
      {
      delete cmd;
      continue;
      }

    double length= d->screenVector.norm();
    if( ! d->growing )
      length= -length;
    Vector t;
    switch( d->button )
      {
      case 3:  t= Vector(1,1,1+length); break;
      case 2:  t= Vector(1,1+length,1); break;
      case 1:
      default: t= Vector(1+length,1,1); break;
      }

    cmd->v= t;

    model->post(cmd);
    }

  model->popMessage();

  module->leaveMode();

  return ITRUE;
}

/////////////////////////////////////////////////////////////////////////////
